% Class for communicating with the Kinect Camera. Uses SharedMemDLL.dll and DepthWithColor-D3D.
classdef Kinect
    %Kinect Interface Functions to communicate with the Kinect Camera
    properties
        h
        w
        rgIndex
        bIndex
        validRead
        dst
        pDst
        pValidRead
    end
    
    methods
        function obj = Kinect()
            obj.h = 480 ;
            obj.w = 640 ;
            obj.rgIndex = obj.w*obj.h+(1:2:obj.w*obj.h*2) ;
            obj.bIndex = obj.w*obj.h+(2:2:obj.w*obj.h*2) ;
            obj.validRead = 0 ;
            obj = obj.Init() ;
        end
        function obj = Init(obj)
            %   Unload library if already loaded first
            if ( libisloaded('SharedMemDLL') )
                unloadlibrary('SharedMemDLL') ;
            end
            % Open SharedMemDLL.dll
            [notfound,warnings]=loadlibrary('\SharedMem\x64\Debug\SharedMemDLL.dll','SharedMem\SharedMemDLL\SharedMemDLL.h') ;
            % Init SharedMemDLL to create shared memory
            retValInit = calllib('SharedMemDLL','InitSharedMem',1,6*obj.h*obj.w,10) ;
            if ( retValInit ~= 0 )
                error(strcat('Probs need to reboot, SharedMemDLL Error:',num2str(retValInit))) ;
            end
            obj.dst = uint16(zeros(1,obj.h*obj.w*3)) ; % Depth and RGB stored in flat array
            obj.pDst = libpointer('uint16Ptr',obj.dst) ;
            obj.pValidRead = libpointer('int32Ptr',obj.validRead) ;
            % Launch Kinect Application that writes frames to the shared memory
            !DepthWithColor-D3D\DepthWithColor-D3D.bat &
            
        end
        function [rgb,depth] = GetImage(obj)
            obj.validRead = 0 ;
            % Wait for new frame
            while ( obj.validRead == 0 )
                [retValRead,obj.dst,obj.validRead] = calllib('SharedMemDLL','ReadSharedMem',obj.pDst,obj.pValidRead) ;
            end
            % Get Depth image
            depth = fliplr(reshape(obj.dst(1,1:obj.w*obj.h),obj.w,obj.h)') ; 
            % Get RGB Image
            rgb(:,:,3) = fliplr(uint8(reshape(bitand(obj.dst(1,obj.rgIndex),255),obj.w,obj.h)')) ;
            rgb(:,:,2) = fliplr(uint8(reshape(obj.dst(1,obj.rgIndex)/256,obj.w,obj.h)')) ;
            rgb(:,:,1) = fliplr(uint8(reshape(bitand(obj.dst(1,obj.bIndex),255),obj.w,obj.h)')) ;
        end
        function ShutDown(obj)
            % Kill (ungracefully Kinect SDK application
            !taskkill /IM DepthWithColor-D3D.exe
            % Shutdown SharedMemDLL
            retValShutdown = calllib('SharedMemDLL','ShutdownSharedMem') ;
            % Unload SharedMemDLL
            unloadlibrary('SharedMemDLL') ;        
        end
    end
    
end
